@@ -20,35 +20,34 @@ class AggregationOp(Enum):
 
 
 class Aggregation:
-
     allow_metric_none = False
 
     def __init__(
         self,
-        metric: Union[str, Metric],
+        dim_or_metric: Union[str, Metric, Dimension],
         op: AggregationOp,
         name: Optional[str] = None,
         distinct: Optional[Union[str, Dimension]] = None,
         filters: Optional[List[Union[Filter, Dict]]] = None,
+        over_all: Optional[bool] = False,
     ):
         """
-        If `filter` is given, it should be applied to each maching row before the aggregation
+        If `filter` is given, it should be applied to each matching row before the aggregation
         and only matching rows should be aggregated.
         """
-        self.metric = metric
+        self.dim_or_metric = dim_or_metric
         self.op: AggregationOp = op
         self.name: str = name if name else op.value
         self.distinct = distinct
         self.filters: List[Union[Filter, Dict]] = filters or []
+        self.over_all = over_all
         self._check_config()
 
     def _check_config(self):
         if self.distinct and self.op not in (AggregationOp.ARRAY, AggregationOp.COUNT):
             raise ConfigurationError(f"Aggregation {self.op.value} does not support `distinct`")
-        if self.distinct and self.metric:
-            raise ConfigurationError(
-                "`distinct` [Dimension] and `metric` [Metric] are mutually exclusive"
-            )
+        if self.distinct and self.dim_or_metric:
+            raise ConfigurationError("`distinct` and `dim_or_metric` are mutually exclusive")
 
 
 class AggregationShorthand(Aggregation):
@@ -60,12 +59,15 @@ class AggregationShorthand(Aggregation):
 
     def __init__(
         self,
-        metric: Union[str, Metric] = None,
+        dim_or_metric: Union[str, Metric, Dimension] = None,
         name: Optional[str] = None,
         distinct: Optional[Union[str, Dimension]] = None,
         filters: Optional[List[Union[Filter, Dict]]] = None,
+        over_all: Optional[bool] = False,
     ):
-        super().__init__(metric, self.agg_op, name, distinct=distinct, filters=filters)
+        super().__init__(
+            dim_or_metric, self.agg_op, name, distinct=distinct, filters=filters, over_all=over_all
+        )
 
 
 class Sum(AggregationShorthand):
