@@ -1,5 +1,6 @@
 import abc
 import ipaddress
+import uuid
 from datetime import date, datetime
 from typing import TYPE_CHECKING, Any, Optional, Type, Union
 
@@ -183,3 +184,8 @@ class IPv4Dimension(IPDimension):
 class IPv6Dimension(IPDimension):
     _type = ipaddress.IPv6Address
     default = "::"
+
+
+class UUIDDimension(Dimension):
+    _type = uuid.UUID
+    default = uuid.UUID("00000000-0000-0000-0000-000000000000")
