@@ -41,7 +41,7 @@ class DictionaryGroupValuesInjection(GroupValuesInjection):
         self.dictionary_keys = dictionary_keys or [g for g in group_by]
 
     def groups_and_keys(self):
-        return list(zip(self.group_by, self.dictionary_keys))
+        return list(zip(self.group_by, self.dictionary_keys, strict=True))
 
 
 class QueryGroupValuesInjection(GroupValuesInjection):
