@@ -16,15 +16,9 @@ from hcube.api.models.aggregation import Aggregation, AggregationOp, ArrayAgg, C
 from hcube.api.models.cube import Cube
 from hcube.api.models.dimensions import (
     ArrayDimension,
-    BooleanDimension,
-    DateDimension,
-    DateTimeDimension,
     Dimension,
     IntDimension,
-    IPv4Dimension,
-    IPv6Dimension,
     MapDimension,
-    StringDimension,
 )
 from hcube.api.models.filters import (
     ComparisonFilter,
@@ -41,7 +35,7 @@ from hcube.api.models.filters import (
 )
 from hcube.api.models.injections import DictionaryGroupValuesInjection, QueryGroupValuesInjection
 from hcube.api.models.materialized_views import AggregatingMaterializedView
-from hcube.api.models.metrics import FloatMetric, IntMetric, Metric
+from hcube.api.models.metrics import IntMetric, Metric
 from hcube.api.models.query import CubeQuery
 from hcube.api.models.transforms import (
     ExplicitMappingTransform,
@@ -55,6 +49,7 @@ from hcube.api.models.transforms import (
 from hcube.backends.clickhouse.data_sources import DataSource
 from hcube.backends.clickhouse.dictionaries import DictionaryDefinition
 from hcube.backends.clickhouse.indexes import IndexDefinition
+from hcube.backends.clickhouse.types import DIMENSION_TYPE_MAP
 from hcube.settings import GlobalSettings
 
 logger = logging.getLogger("hcube.backends.clickhouse")
@@ -114,17 +109,6 @@ class ClickhouseCubeBackend(CubeBackend):
     Backend to Clickhouse using the low-level Clickhouse API from `clickhouse-driver`.
     """
 
-    dimension_type_map = (
-        (IntDimension, "Int"),
-        (StringDimension, "String"),
-        (DateDimension, "Date"),
-        (DateTimeDimension, "DateTime64"),
-        (FloatMetric, "Float"),
-        (IntMetric, "Int"),
-        (IPv4Dimension, "IPv4"),
-        (IPv6Dimension, "IPv6"),
-        (BooleanDimension, "Bool"),
-    )
     default_settings = {}
 
     def __init__(
@@ -172,13 +156,18 @@ class ClickhouseCubeBackend(CubeBackend):
         self._drop_table(cube)
         self._table_exists.pop(self.cube_to_table_name(cube), None)
 
-    def sync_storage(self, cube: Type[Cube], drop=False) -> (bool, Set[str], Set[str]):
+    def sync_storage(
+        self, cube: Type[Cube], drop=False, soft_sync=False, recreate=False
+    ) -> (bool, Set[str], Set[str]):
         """
         Assumes that the table already exists and makes sure columns are added or removed
         to match the cube definition.
 
         :param cube: the cube class
         :param drop: if True, drop columns that are not in the cube definition
+        :param soft_sync: if True, only add or remove columns, but do not reorder them or update
+        comments
+        :param recreate: if True, recreate the table if it cannot be synced without dropping
         :returns: (changed, added, removed) where changed is True if the table was changed;
         added is a set of columns added; removed is a set of columns to be removed - regardless
         of whether they were actually removed or not (depends on `drop`).
@@ -193,31 +182,72 @@ class ClickhouseCubeBackend(CubeBackend):
             cols_to_add = cube_col_names - db_col_names
             cols_to_remove = db_col_names - cube_col_names
 
+            # first remove obsolete columns if requested
             changed = False
             if drop and cols_to_remove:
-                logger.debug(f"Dropping columns {cols_to_add} from table '{table_name}'")
+                # get the sorting key of the table to check if the dropped columns are in it
+                sorting_key = client.execute(
+                    "SELECT sorting_key FROM system.tables WHERE "
+                    "name=%(name)s AND database=%(database)s",
+                    {"name": self.cube_to_table_name(cube), "database": self.database},
+                )[0][0]
+                sorting_key_cols = [col.strip() for col in sorting_key.split(", ")]
+                dropped_sorting_columns = [col for col in cols_to_remove if col in sorting_key_cols]
+                if dropped_sorting_columns:
+                    # sorting key columns cannot be dropped, so we need to recreate the table
+                    # but only if `recreate` is True - this is to avoid accidental data loss
+                    if recreate:
+                        self.drop_storage(cube)
+                        self.initialize_storage(cube)
+                        # this deals will all the other changes, so we can return early
+                        return True, cols_to_add, cols_to_remove
+                    else:
+                        raise ConfigurationError(
+                            f"Dropping columns {dropped_sorting_columns} from table '{table_name}' "
+                            "is not supported because they are in the sorting key. "
+                            "Use the `recreate` parameter to drop and recreate the table."
+                        )
+                logger.debug(f"Dropping columns {cols_to_remove} from table '{table_name}'")
                 for col in cols_to_remove:
                     client.execute(f"ALTER TABLE {table_name} DROP COLUMN {col}")
                 changed = True
 
+            # add new columns
             if cols_to_add:
                 logger.debug(f"Adding columns {cols_to_add} to table '{table_name}'")
                 # we add columns in a way that preserves the order of the columns in the definition
                 last_col = None
                 logger.debug("Cube dimensions: %s", cube._dimensions)
-                for name, dim in cube._dimensions.items():
+                for name, dim in itertools.chain(cube._dimensions.items(), cube._metrics.items()):
                     if name in cols_to_add:
                         spec = self._dim_spec(dim)
                         pos = f"AFTER {last_col}" if last_col else "FIRST"
                         client.execute(f"ALTER TABLE {table_name} ADD COLUMN {spec} {pos}")
                     last_col = name
-                for name, metric in cube._metrics.items():
-                    if name in cols_to_add:
-                        spec = self._dim_spec(metric)
-                        pos = f"AFTER {last_col}" if last_col else "FIRST"
-                        client.execute(f"ALTER TABLE {table_name} ADD COLUMN {spec} {pos}")
-                    last_col = name
                 changed = True
+
+            # reorder existing columns
+            # all the columns are guaranteed to be in the db, so we can just reorder them
+            if not soft_sync:
+                # [0] is the name, [4] is the comment
+                db_col_order = [(col[0], col[4]) for col in db_cols if col[0] not in cols_to_remove]
+                cube_col_order = [
+                    (col.name, col.help_text)
+                    for col in itertools.chain(cube._dimensions.values(), cube._metrics.values())
+                ]
+                # check if either the order or the comments are different
+                if db_col_order != cube_col_order:
+                    logger.debug(f"Reordering columns in table '{table_name}'")
+                    last_col = None
+                    for name, dim in itertools.chain(
+                        cube._dimensions.items(), cube._metrics.items()
+                    ):
+                        spec = self._dim_spec(dim)
+                        pos = f"AFTER {last_col}" if last_col else "FIRST"
+                        client.execute(f"ALTER TABLE {table_name} MODIFY COLUMN {spec} {pos}")
+                        last_col = name
+                    changed = True
+
             return changed, cols_to_add, cols_to_remove
 
     def _column_names(self, cube: Type[Cube]) -> [str]:
@@ -235,7 +265,7 @@ class ClickhouseCubeBackend(CubeBackend):
     def store_records(
         self,
         cube: Type[Cube],
-        records: Iterable[NamedTuple | Dict],
+        records: Iterable[Union[NamedTuple, Dict]],
         skip_cleanup: bool = False,
         dict_records: bool = False,
     ):
@@ -355,7 +385,7 @@ class ClickhouseCubeBackend(CubeBackend):
             if meta.lightweight_delete_mode:
                 whole_text += (
                     f" SETTINGS lightweight_delete_mode='{meta.lightweight_delete_mode}',"
-                    "allow_experimental_lightweight_update=1"
+                    "allow_experimental_lightweight_update=1,enable_lightweight_update=1"
                 )
         else:
             # if we do not use lightweight deletes, we must insert records with the opposite sign
@@ -960,6 +990,8 @@ class ClickhouseCubeBackend(CubeBackend):
 
     def _dim_spec(self, dim) -> str:
         field = f"{dim.name} {self._ch_type(dim)}"
+        if hasattr(dim, "help_text") and dim.help_text:
+            field += f" COMMENT '{dim.help_text}'"
         if ch_spec := dim.kwargs.get("clickhouse"):
             if codec := ch_spec.get("compression_codec"):
                 field += f" CODEC({codec}, LZ4)"
@@ -973,7 +1005,7 @@ class ClickhouseCubeBackend(CubeBackend):
             key_subtype = self._ch_type(dimension.key_dimension)
             value_subtype = self._ch_type(dimension.value_dimension)
             return f"Map({key_subtype}, {value_subtype})"
-        for dim_cls, ch_type in self.dimension_type_map:
+        for dim_cls, ch_type in DIMENSION_TYPE_MAP.items():
             if isinstance(dimension, (IntDimension, IntMetric)):
                 sign = "U" if not dimension.signed else ""
                 ch_type = f"{sign}{ch_type}{dimension.bits}"
