@@ -4,42 +4,75 @@ from dataclasses import dataclass
 from typing import List, Optional, Union
 
 from hcube.api.models.cube import Cube
-from hcube.api.models.dimensions import IntDimension, StringDimension
+from hcube.api.models.dimensions import (
+    Dimension,
+    IntDimension,
+    StringDimension,
+)
 from hcube.backends.clickhouse.data_sources import DataSource
+from hcube.backends.clickhouse.types import DIMENSION_TYPE_MAP, TYPE_TO_DIMENSION
 
 
 @dataclass
 class DictionaryAttr:
     name: str
-    type: str
+    type: Union[str, Dimension]
     expression: Optional[str] = None
     null_value: str = "NULL"
     injective: bool = False
 
+    def __post_init__(self):
+        if isinstance(self.type, str):
+            try:
+                self.type = TYPE_TO_DIMENSION[self.type]()
+            except KeyError:
+                raise ValueError(f"Invalid type: {self.type}") from None
+
     def definition_sql(self):
         expression = f"EXPRESSION {self.expression}" if self.expression else ""
         default = f"DEFAULT {self.null_value}" if self.null_value else ""
-        type_part = f"Nullable({self.type})" if self.null_value == "NULL" else self.type
+        type_part = (
+            f"Nullable({self.clickhouse_type})"
+            if self.null_value == "NULL"
+            else self.clickhouse_type
+        )
         return (
             f"{self.name} {type_part} {default} {expression} "
             f"{'INJECTIVE' if self.injective else ''}"
         )
 
+    @property
+    def clickhouse_type(self):
+        return DIMENSION_TYPE_MAP[self.type.__class__]
+
+    def to_json(self):
+        return {
+            "kind": "attr",
+            "name": self.name,
+            "type": self.type.__class__.__name__,
+            "expression": self.expression,
+            "null_value": self.null_value,
+            "injective": self.injective,
+        }
+
 
 class DictionaryDefinition:
     def __init__(
         self,
         name: str,
         source: DataSource,
-        key: Union[str, List[str]],
+        key: Union[str, DictionaryAttr, List[Union[str, DictionaryAttr]]],
         layout: str,
         attrs: [DictionaryAttr],
         lifetime_min: int = 600,
         lifetime_max: int = 720,
     ):
         self.name = name
-        # Internally store keys as a list; single key is converted for backward compatibility.
-        self.keys: List[str] = [key] if isinstance(key, str) else list(key)
+        # Normalize keys to a list
+        if isinstance(key, (str, DictionaryAttr)):
+            self.keys: List[Union[str, DictionaryAttr]] = [key]
+        elif isinstance(key, list):
+            self.keys = key
         self.source = source
         self.layout = layout
         self.attrs = attrs
@@ -48,15 +81,26 @@ class DictionaryDefinition:
 
     def definition_sql(self, database=None):
         db_part = f"{database}." if database else ""
-        # Columns: always auto-define all key columns as UInt64
-        columns = [f"{k} UInt64" for k in self.keys]
-        columns.extend([attr.definition_sql() for attr in self.attrs])
+        columns = []
+        for k in self.keys:
+            if isinstance(k, DictionaryAttr):
+                # use the type from the DictionaryAttr
+                columns.append(f"{k.name} {k.clickhouse_type}")
+            elif isinstance(k, str):
+                # when the key is just a string, use UInt64 type for the column
+                columns.append(f"{k} UInt64")
+            else:
+                raise ValueError(f"Invalid key type: {k}")
+        for attr in self.attrs:
+            columns.append(attr.definition_sql())
+
         cols_sql = ",\n".join(columns)
         # primary key
         if len(self.keys) == 1:
-            pk_sql = f"PRIMARY KEY {self.keys[0]}"
+            pk = self.keys[0]
+            pk_sql = f"PRIMARY KEY {pk.name if isinstance(pk, DictionaryAttr) else pk}"
         else:
-            joined = ", ".join(self.keys)
+            joined = ", ".join(k.name if isinstance(k, DictionaryAttr) else k for k in self.keys)
             pk_sql = f"PRIMARY KEY ({joined})"
         return (
             f"CREATE DICTIONARY IF NOT EXISTS {db_part}{self.name} ("
@@ -71,9 +115,14 @@ class DictionaryDefinition:
 
     @property
     def checksum(self):
+        def _serialize_key(k: Union[str, "DictionaryAttr"]):
+            if isinstance(k, str):
+                return {"kind": "string", "name": k}
+            return k.to_json()
+
         data = {
             "name": self.name,
-            "key": self.keys,
+            "key": [_serialize_key(k) for k in self.keys],
             "source": self.source.definition_sql(),
             "layout": self.layout,
             "attrs": [attr.definition_sql() for attr in self.attrs],
@@ -91,9 +140,14 @@ class DictionaryDefinition:
             class Clickhouse:
                 source = self
 
-        # Key columns are numeric (UInt64)
+        # Key columns: infer dimension from key type
         for k in self.keys:
-            setattr(Out, k, IntDimension())
+            if isinstance(k, DictionaryAttr):
+                setattr(Out, k.name, k.clickhouse_type)
+            elif isinstance(k, str):
+                setattr(Out, k, IntDimension())
+            else:
+                raise ValueError(f"Invalid key type: {k}")
         for attr in self.attrs:
             setattr(Out, attr.name, StringDimension())
 
