@@ -10,6 +10,5 @@ class IndexDefinition:
 
     def definition(self) -> str:
         return (
-            f"INDEX {self.name} ({self.expression}) TYPE {self.type} GRANULARITY "
-            f"{self.granularity}"
+            f"INDEX {self.name} ({self.expression}) TYPE {self.type} GRANULARITY {self.granularity}"
         )
